@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012-2013  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2012-2017  Kouhei Sutou <kou@clear-code.com>
 # Copyright (C) 2010  masone (Christian Felder) <ema@rh-productions.ch>
 # Copyright (C) 2009  Masao Mutoh
 #
@@ -58,7 +58,9 @@ module GetText
     attr_accessor :references    # ["file1:line1", "file2:line2", ...]
     attr_accessor :translator_comment
     attr_accessor :extracted_comment
-    attr_accessor :flag
+    # @return [Array<String>] The flags for this PO entry.
+    # @since 3.0.4
+    attr_accessor :flags
     attr_accessor :previous
     attr_accessor :comment
 
@@ -68,7 +70,7 @@ module GetText
       @translator_comment = nil
       @extracted_comment = nil
       @references = []
-      @flag = nil
+      @flags = []
       @previous = nil
       @msgctxt = nil
       @msgid = nil
@@ -81,12 +83,30 @@ module GetText
     # @return [void]
     def add_comment(new_comment)
       if (new_comment and ! new_comment.empty?)
-        @extracted_comment ||= ""
+        @extracted_comment ||= String.new
         @extracted_comment << "\n" unless @extracted_comment.empty?
         @extracted_comment << new_comment
       end
     end
 
+    # @return [String, nil] The flag of the PO entry.
+    # @deprecated Since 3.0.4. Use {#flags} instead.
+    def flag
+      @flags.first
+    end
+
+    # Set the new flag for the PO entry.
+    #
+    # @param [String, nil] flag The new flag.
+    # @deprecated Since 3.0.4. Use {#flags=} instead.
+    def flag=(flag)
+      if flag.nil?
+        @flags = []
+      else
+        @flags = [flag]
+      end
+    end
+
     # Checks if the self has same attributes as other.
     def ==(other)
       not other.nil? and
@@ -99,7 +119,7 @@ module GetText
         translator_comment == other.translator_comment and
         extracted_comment == other.extracted_comment and
         references == other.references and
-        flag == other.flag and
+        flags == other.flags and
         previous == other.previous and
         comment == other.comment
     end
@@ -177,6 +197,19 @@ module GetText
       @type == :normal and @msgid == :last
     end
 
+    # @return true if the entry is fuzzy entry, false otherwise.
+    #   Fuzzy entry has "fuzzy" flag.
+    def fuzzy?
+      @flags.include?("fuzzy")
+    end
+
+    # @return true if the entry is translated entry, false otherwise.
+    def translated?
+      return false if fuzzy?
+      return false if @msgstr.nil? or @msgstr.empty?
+      true
+    end
+
     def [](number)
       param = @param_type[number]
       raise ParseError, 'no more string parameters expected' unless param
@@ -218,8 +251,22 @@ module GetText
 
       # @param [POEntry] entry The entry to be formatted.
       # @param [Hash] options
+      # @option options [Bool] :include_translator_comment (true)
+      #   Includes translator comments in formatted string if true.
+      # @option options [Bool] :include_extracted_comment (true)
+      #   Includes extracted comments in formatted string if true.
       # @option options [Bool] :include_reference_comment (true)
       #   Includes reference comments in formatted string if true.
+      # @option options [Bool] :include_flag_comment (true)
+      #   Includes flag comments in formatted string if true.
+      # @option options [Bool] :include_previous_comment (true)
+      #   Includes previous comments in formatted string if true.
+      # @option options [Bool] :include_all_comments (true)
+      #   Includes all comments in formatted string if true.
+      #   Other specific `:include_XXX` options get preference over
+      #   this option.
+      #   You can remove all comments by specifying this option as
+      #   false and omitting other `:include_XXX` options.
       # @option options [Integer] :max_line_width (78)
       #   Wraps long lines that is longer than the `:max_line_width`.
       #   Don't break long lines if `:max_line_width` is less than 0
@@ -228,7 +275,7 @@ module GetText
       #   Encodes to the specific encoding.
       def initialize(entry, options={})
         @entry = entry
-        @options = fill_default_option_values(options)
+        @options = normalize_options(options)
       end
 
       def format
@@ -236,14 +283,7 @@ module GetText
           return format_obsolete_comment(@entry.comment)
         end
 
-        str = ""
-        str << format_translator_comment
-        str << format_extracted_comment
-        if @options[:include_reference_comment]
-          str << format_reference_comment
-        end
-        str << format_flag_comment
-        str << format_previous_comment
+        str = format_comments
 
         # msgctxt, msgid, msgstr
         if @entry.msgctxt?
@@ -285,15 +325,66 @@ module GetText
       end
 
       private
-      def fill_default_option_values(options)
+      def normalize_options(options)
         options = options.dup
-        if options[:include_reference_comment].nil?
-          options[:include_reference_comment] = true
+        include_comment_keys = [
+          :include_translator_comment,
+          :include_extracted_comment,
+          :include_reference_comment,
+          :include_flag_comment,
+          :include_previous_comment,
+        ]
+        if options[:include_all_comments].nil?
+          options[:include_all_comments] = true
+        end
+        default_include_comment_value = options[:include_all_comments]
+        include_comment_keys.each do |key|
+          options[key] = default_include_comment_value if options[key].nil?
         end
         options[:max_line_width] ||= DEFAULT_MAX_LINE_WIDTH
         options
       end
 
+      def include_translator_comment?
+        @options[:include_translator_comment]
+      end
+
+      def include_extracted_comment?
+        @options[:include_extracted_comment]
+      end
+
+      def include_reference_comment?
+        @options[:include_reference_comment]
+      end
+
+      def include_flag_comment?
+        @options[:include_flag_comment]
+      end
+
+      def include_previous_comment?
+        @options[:include_previous_comment]
+      end
+
+      def format_comments
+        formatted_comment = String.new
+        if include_translator_comment?
+          formatted_comment << format_translator_comment
+        end
+        if include_extracted_comment?
+          formatted_comment << format_extracted_comment
+        end
+        if include_reference_comment?
+          formatted_comment << format_reference_comment
+        end
+        if include_flag_comment?
+          formatted_comment << format_flag_comment
+        end
+        if include_previous_comment?
+          formatted_comment << format_previous_comment
+        end
+        formatted_comment
+      end
+
       def format_translator_comment
         format_comment("#", @entry.translator_comment)
       end
@@ -304,7 +395,7 @@ module GetText
 
       def format_reference_comment
         max_line_width = @options[:max_line_width]
-        formatted_reference = ""
+        formatted_reference = String.new
         if not @entry.references.nil? and not @entry.references.empty?
           formatted_reference << REFERENCE_COMMENT_MARK
           line_width = 2
@@ -326,7 +417,11 @@ module GetText
       end
 
       def format_flag_comment
-        format_comment(FLAG_MARK, @entry.flag)
+        formatted_flags = String.new
+        @entry.flags.each do |flag|
+          formatted_flags << format_comment(FLAG_MARK, flag)
+        end
+        formatted_flags
       end
 
       def format_previous_comment
@@ -336,7 +431,7 @@ module GetText
       def format_comment(mark, comment)
         return "" if comment.nil?
 
-        formatted_comment = ""
+        formatted_comment = String.new
         comment.each_line do |comment_line|
           if comment_line == "\n"
             formatted_comment << "#{mark}\n"
@@ -351,7 +446,7 @@ module GetText
         mark = "#~"
         return "" if comment.nil?
 
-        formatted_comment = ""
+        formatted_comment = String.new
         comment.each_line do |comment_line|
           if /\A#[^~]/ =~ comment_line or comment_line.start_with?(mark)
             formatted_comment << "#{comment_line.chomp}\n"
@@ -365,11 +460,16 @@ module GetText
       end
 
       def format_message(message)
-        return "\"\"\n" if message.nil?
+        empty_formatted_message = "\"\"\n"
+        return empty_formatted_message if message.nil?
 
         chunks = wrap_message(message)
-        formatted_message = ""
-        formatted_message << "\"\"\n" if chunks.size > 1
+        return empty_formatted_message if chunks.empty?
+
+        formatted_message = String.new
+        if chunks.size > 1 or chunks.first.end_with?("\n")
+          formatted_message << empty_formatted_message
+        end
         chunks.each do |chunk|
           formatted_message << "\"#{escape(chunk)}\"\n"
         end
